@@ -1,21 +1,22 @@
 # -*- coding:utf-8 -*- 
 
 from BDMLtools.clearner import outliersTransformer,dtypeAllocator,nanTransformer,dtStandardization
-from BDMLtools.report import businessReport,EDAReport,varReport,varGroupsReport,varReportSinge,varGroupsPlot
+from BDMLtools.report import businessReport,EDAReport,varReport,varGroupsReport,varReportSinge,varGroupsPlot,GainsTable
 from BDMLtools.selector import binSelector,binAdjuster,binFreq,binPretty,binTree,binChi2,binKmeans
 from BDMLtools.selector import faSelector
 from BDMLtools.selector import stepLogit,cardScorer
 from BDMLtools.selector import preSelector,prefitModel
+from BDMLtools.selector import LgbmSeqSelector,LgbmPISelector,LgbmShapRFECVSelector
 #from BDMLtools.selector import RFECVSelector
 from BDMLtools.selector import lassoSelector
 from BDMLtools.plotter import BaseWoePlotter
 from BDMLtools.plotter import perfEval
 from BDMLtools.encoder import woeTransformer
-from BDMLtools.tuner import girdTuner,hgirdTuner
-from BDMLtools.tuner import BayesianXGBTuner,BayesianLgbmTuner,shapCheck
+from BDMLtools.tuner import gridTuner,hgridTuner
+from BDMLtools.tuner import BayesianCVTuner,shapCheck
 
 
-__version__ = '0.2.0'
+__version__ = '0.2.3'
 
 __all__ = (    
     dtStandardization,
@@ -29,6 +30,7 @@ __all__ = (
     varReport,
     varGroupsReport,
     varGroupsPlot,
+    GainsTable,
     binSelector,
     binAdjuster,
     binFreq,
@@ -42,11 +44,13 @@ __all__ = (
     cardScorer,
     preSelector,
     lassoSelector,
+    LgbmSeqSelector,
+    LgbmPISelector,
+    LgbmShapRFECVSelector,
     woeTransformer,
-    girdTuner,
-    hgirdTuner,
-    BayesianXGBTuner,
-    BayesianLgbmTuner,
+    gridTuner,
+    hgridTuner,
+    BayesianCVTuner,
     perfEval,
     shapCheck
 )
