@@ -13,6 +13,7 @@ from pandas.api.types import is_array_like
 from warnings import warn
 import pandas as pd
 import numpy as np
+from itertools import combinations
 
 class Base:
     
@@ -24,6 +25,23 @@ class Base:
                                  "'fit' with appropriate arguments."
                                  .format(self.__class__.__name__))
             
+    def _check_x(self,x):
+        
+        if not isinstance(x,pd.core.frame.Series):
+            
+            raise DataTypeError("x is pd.core.frame.Series")
+            
+
+    def _check_ind(self,lst):
+        
+        if not np.all([np.all(np.equal(com[0],com[1])) for com in combinations([x.index for x in lst],2)]):
+            
+            raise XyIndexError("All data's index must be equal")
+            
+        if not np.all([(x.index.is_unique) for x in lst]):
+            
+            raise XyIndexError("All data's index must be unique")      
+            
             
     def _check_X(self,X):
  
@@ -35,6 +53,12 @@ class Base:
                 
             raise XyIndexError("X.index is not unique")
             
+    def _check_colname(self,X):
+        
+        if not X.columns.is_unique:
+            
+            raise XyIndexError("duplicated colname in X")
+            
             
     def _check_data(self,X,y):
         
@@ -60,7 +84,20 @@ class Base:
 
         if not np.isin(y.unique(),[0,1]).any():
         
-            raise yValueError("vals of y in [0,1] and 0(no-event),1(event)")                
+            raise yValueError("vals of y in [0,1] and 0(no-event),1(event)")    
+
+
+    def _check_ws(self,y,sample_weight):
+
+        if sample_weight is not None:
+            
+            if not isinstance(sample_weight,pd.Series):
+                
+                raise DataTypeError("sample_weight is not pandas.Series.") 
+                
+            if not y.index.equals(sample_weight.index):
+                
+                raise XyIndexError("index of sample_weight not equal to y_pred or y_true")             
 
     
     def _check_param_dtype(self,dtype):
