@@ -21,4 +21,12 @@ class XyIndexError(IndexError):
     
     """Exception class to raise if X's index not equal to y's index
 
-    """ 
\ No newline at end of file
+    """ 
+    
+class yValueError(ValueError):
+    
+    """Exception class to raise if y values not equal to number 0 or number 1
+
+    """ 
+    
+    
\ No newline at end of file
