@@ -7,23 +7,24 @@ Created on Wed Jan 12 17:46:24 2022
 """
 
 from BDMLtools.clearner import outliersTransformer,dtypeAllocator,nanTransformer,dtStandardization
-from BDMLtools.report import businessReport,EDAReport,varReport,varGroupsReport,varReportSinge
+from BDMLtools.report import businessReport,EDAReport,varReport,varGroupsReport,varReportSinge,GainsTable
 from BDMLtools.selector import binSelector,binFreq,binPretty,binTree,binChi2,binKmeans
 from BDMLtools.selector import faSelector
 from BDMLtools.selector import stepLogit,cardScorer
 from BDMLtools.selector import preSelector,prefitModel
 #from BDMLtools.selector import RFECVSelector
-from BDMLtools.selector import lassoSelector,LgbmRFESelector,LgbmSeqSelector
+from BDMLtools.selector import lassoSelector,LgbmSeqSelector,LgbmShapRFECVSelector
 from BDMLtools.plotter import  perfEval
 from BDMLtools.encoder import woeTransformer
-from BDMLtools.tuner import girdTuner,hgirdTuner
-from BDMLtools.tuner import BayesianXGBTuner,BayesianLgbmTuner,shapCheck
+from BDMLtools.tuner import gridTuner,hgridTuner
+from BDMLtools.tuner import shapCheck,BayesianCVTuner
 import scorecardpy as sc
 from sklearn.linear_model import LogisticRegression 
 import pandas as pd
 import numpy as np
 from xgboost import XGBClassifier
 from lightgbm import LGBMClassifier
+from catboost import CatBoostClassifier
 
 
 class test:
@@ -40,8 +41,8 @@ class test:
         self.test_scorecard()
         self.test_perfEval()
         self.test_tab()
-        self.test_tunner()
-        
+        self.test_tunner()     
+        self.test_perfEval()
 
     def test_dtStandardization(self):
     
@@ -103,20 +104,15 @@ class test:
         
         
         dtypes_dict={
-            'num':['age.in.years',
-                 'credit.amount',
-                 'creditability',
-                 'duration.in.month',
+            'float':[                 
                  'installment.rate.in.percentage.of.disposable.income',
                  'number.of.existing.credits.at.this.bank',
                  'number.of.people.being.liable.to.provide.maintenance.for',
                  'present.residence.since',
                  'floattest',
-                 'booltest',
-                 ],
-            'str':[
-                   #'housing',
-                   #'telephone',
+                 ],            
+            'int':['age.in.years','creditability','duration.in.month','booltest'],
+            'str':['credit.amount',
                    'foreign.worker','purpose','job','personal.status.and.sex','property',
                    'credit.history','savings.account.and.bonds','present.employment.since',
                    'status.of.existing.checking.account',
@@ -125,11 +121,10 @@ class test:
             'tdiff':['datediff']
         }
         
-        da=dtypeAllocator(dtypes_dict=dtypes_dict,col_rm=['housing','telephone'],t_unit='1 D',dtype_num='float64',
+        da=dtypeAllocator(dtypes_dict=dtypes_dict,col_rm=['housing','telephone'],t_unit='1 D',dtype_num='64',
                                                           drop_date=False,precision=3).fit(dt)
         dt_1=da.transform(dt)
         
-       
 
         if not np.equal(dt_1[['housing','telephone']].dtypes,['category','category']).all():
             
@@ -145,17 +140,14 @@ class test:
             
 
         dtypes_dict={
-            'num':['age.in.years',
-                 'credit.amount',
-                 'creditability',
-                 'duration.in.month',
+            'float':[                 
                  'installment.rate.in.percentage.of.disposable.income',
                  'number.of.existing.credits.at.this.bank',
                  'number.of.people.being.liable.to.provide.maintenance.for',
                  'present.residence.since',
                  'floattest',
-                 'booltest',
-                 ],
+                 ],            
+            'int':['age.in.years','creditability','duration.in.month','booltest'],
             'str':[
                    'housing',
                    'telephone',
@@ -167,7 +159,7 @@ class test:
             'tdiff':['datediff']
         }
         
-        da=dtypeAllocator(dtypes_dict=dtypes_dict,col_rm=None,t_unit='15 D',dtype_num='float64',
+        da=dtypeAllocator(dtypes_dict=dtypes_dict,col_rm=None,t_unit='15 D',dtype_num='64',
                                                           drop_date=True,precision=3).fit(dt)
         dt_1=da.transform(dt)
 
@@ -175,32 +167,29 @@ class test:
             
             raise ValueError('param drop_date in dtypeAllocator error')
             
-        if not np.equal(dt_1.dtypes.unique(),[np.dtype('float64'),np.dtype('O')]).all():
+        if not np.equal(dt_1.dtypes.unique(),[np.dtype('int64'),np.dtype('float64'),np.dtype('O')]).all():
 
-            raise ValueError('dtypes of out contain unknown type')
+            raise ValueError('dtypes of out contain unknown type')                        
             
-            
-            
-            
-        da=dtypeAllocator(dtypes_dict={},col_rm=None,t_unit='1 D',dtype_num='float64',
+        da=dtypeAllocator(dtypes_dict={},col_rm=None,t_unit='1 D',dtype_num='64',
                                                           drop_date=True,precision=3).fit(dt)
         dt_1=da.transform(dt)  
 
-        if not np.equal(dt_1.dtypes.unique(),[np.dtype('float64'),np.dtype('O')]).all():
+        if not np.equal(dt_1.dtypes.unique(),[np.dtype('int64'),np.dtype('float64'),np.dtype('O')]).all():
 
             raise ValueError('dtypes of out contain unknown type')             
             
             
-        da=dtypeAllocator(dtypes_dict={},col_rm=None,t_unit='1 D',dtype_num='float32',
+        da=dtypeAllocator(dtypes_dict={},col_rm=None,t_unit='1 D',dtype_num='32',
                                                           drop_date=True,precision=3).fit(dt)
         dt_1=da.transform(dt)  
 
-        if not np.equal(dt_1.dtypes.unique(),[np.dtype('float32'),np.dtype('O')]).all():
+        if not np.equal(dt_1.dtypes.unique(),[np.dtype('int32'),np.dtype('float32'),np.dtype('O')]).all():
 
             raise ValueError('dtypes of out contain unknown type') 
 
 
-        da=dtypeAllocator(dtypes_dict={},col_rm=None,t_unit='1 D',dtype_num='float64',
+        da=dtypeAllocator(dtypes_dict={},col_rm=None,t_unit='1 D',dtype_num='64',
                                                           drop_date=True,precision=10).fit(dt)
         dt_1=da.transform(dt)  
 
@@ -209,10 +198,10 @@ class test:
         
             raise ValueError('param precision not work') 
 
-        print('dtypeAllocator test successfully')
-        
-        
-        
+        print('dtypeAllocator test successfully')            
+
+
+
     def test_outliersTransformer(self):
         
         dt=sc.germancredit().copy()
@@ -307,7 +296,7 @@ class test:
                    variance=None,
                    chif_pvalue=None,
                    tree_imps=None,
-                   auc_limit=None,
+                   pi_limit=None,
                    iv_limit=None).fit(X[['na_test']],y).keep_col
 
         if 'na_test' in keep_col:
@@ -319,7 +308,7 @@ class test:
                            variance=None,
                            chif_pvalue=None,
                            tree_imps=None,
-                           auc_limit=None,
+                           pi_limit=None,
                            iv_limit=None).fit(X[['uni_test']],y).keep_col
         
         if not 'uni_test' in keep_col:
@@ -331,7 +320,7 @@ class test:
                            variance=0,
                            chif_pvalue=None,
                            tree_imps=None,
-                           auc_limit=None,
+                           pi_limit=None,
                            iv_limit=None).fit(X[['var_test']],y).keep_col
         
         if 'var_test' in keep_col:
@@ -343,7 +332,7 @@ class test:
                            variance=None,
                            chif_pvalue=0.05,
                            tree_imps=None,
-                           auc_limit=None,
+                           pi_limit=None,
                            iv_limit=None).fit(X[['chi2_test','f_test']],y).keep_col
         
         if 'chi2_test' in keep_col or 'f_test' in keep_col:
@@ -612,42 +601,12 @@ class test:
         
         lassoSelector().fit(dt_woe_bm,y)
         
-        LgbmRFESelector().fit(dt_woe_bm,y)
-        
         LgbmSeqSelector().fit(dt_woe_bm,y)
         
         
-        #Bayesian based tunner
-        BayesianXGBTuner(para_space={
-                     'n_estimators': (80, 150),
-                     'learning_rate': (0.05, 0.2),
-                     'max_depth': (3, 10),
-                     'gamma': (0, 20),
-                     'min_child_weight': (0, 10),
-                     'max_delta_step': (0, 0),
-                     'scale_pos_weight': (11,11),
-                     'subsample': (0.5, 1),
-                     'colsample_bytree': (0.5, 1),
-                     'reg_lambda': (0, 10)
-                               }).fit(dt_woe_bm,y)
         
-        BayesianLgbmTuner(para_space={
-                         'boosting_type':'gbdt', 
-                         'n_estimators':(30,120),
-                         'learning_rate':(0.05,0.2), 
-                        
-                         'max_depth':(2,4),
-                         'min_split_gain': (0,20),
-                         'min_sum_hessian_in_leaf': (0,20),
-                         
-                         'scale_pos_weight':(1,1),
-                         'subsample':(0.5,1),
-                         'colsample_bytree' :(0.5,1),
-                         'reg_lambda':(0,10), 
-                         }).fit(dt_woe_bm,y)
-        
-        #Gird based tunner
-        girdTuner(XGBClassifier,para_space={
+        #Grid based tunner
+        gridTuner(XGBClassifier,para_space={
                              'n_estimators':[100],
                              'learning_rate':[0.1],
                             
@@ -660,14 +619,26 @@ class test:
                              'reg_lambda':[0,10], 
                              'scale_pos_weight':[1,10],
                              'max_delta_step':[0]
-                             },method='gird'           
+                             },method='grid'           
             ).fit(dt_woe_bm,y)
         
         
         from scipy.stats import randint as sp_randint
         from scipy.stats import uniform as sp_uniform 
+        from BDMLtools.tuner.base import sLGBMClassifier
         
-        girdTuner(XGBClassifier,para_space={
+        
+        
+        BayesianCVTuner(XGBClassifier,scoring='roc_auc',eval_metric='auc',early_stopping_rounds=10).fit(dt_woe_bm,y)
+        
+        
+        BayesianCVTuner(LGBMClassifier,scoring='roc_auc',eval_metric='auc',early_stopping_rounds=10).fit(dt_woe_bm,y)
+        
+        
+        BayesianCVTuner(CatBoostClassifier,scoring='roc_auc',eval_metric='auc',early_stopping_rounds=10).fit(dt_woe_bm,y)
+        
+        
+        gridTuner(XGBClassifier,para_space={
                   'n_estimators':sp_randint(low=60,high=120),#迭代次数
                   'learning_rate':sp_uniform(loc=0.05,scale=0.15), #学习率
                  
@@ -681,11 +652,11 @@ class test:
                   'reg_lambda':sp_randint(low=0,high=1), 
                   'scale_pos_weight':sp_uniform(loc=1,scale=0), 
                   'max_delta_step':sp_uniform(loc=0,scale=0)
-                  } ,method='random_gird'           
+                  } ,method='random_grid',early_stopping_rounds=10           
             ).fit(dt_woe_bm,y)        
         
         
-        girdTuner(LGBMClassifier,para_space={
+        gridTuner(LGBMClassifier,para_space={
                      'boosting_type':['gbdt','goss'], 
                      'n_estimators':[100],
                      'learning_rate':[0.1], 
@@ -698,11 +669,11 @@ class test:
                      'subsample':[0.6,0.8],
                      'colsample_bytree' :[0.6,0.8],
                      'reg_lambda':[0,10], 
-                             },method='gird'           
+                             },method='grid',early_stopping_rounds=10               
             ).fit(dt_woe_bm,y)
 
         
-        girdTuner(LGBMClassifier,para_space={
+        gridTuner(LGBMClassifier,para_space={
                      'boosting_type':['gbdt','goss'], #'goss','gbdt'
                      'n_estimators':sp_randint(low=100,high=110),
                      'learning_rate':sp_uniform(loc=0.1,scale=0), 
@@ -716,11 +687,11 @@ class test:
                      'colsample_bytree' :sp_uniform(loc=0.5,scale=0.5),
                      'reg_lambda':sp_uniform(loc=0,scale=20),
 
-                  } ,method='random_gird'           
+                  } ,method='random_grid',early_stopping_rounds=10               
             ).fit(dt_woe_bm,y)        
         
         #Halving based tunner
-        hgirdTuner(XGBClassifier,para_space={
+        hgridTuner(XGBClassifier,para_space={
                              'n_estimators':[100],
                              'learning_rate':[0.1],
                             
@@ -733,11 +704,11 @@ class test:
                              'reg_lambda':[0,10], 
                              'scale_pos_weight':[1,10],
                              'max_delta_step':[0]
-                             },method='h_gird'           
+                             },method='h_grid',early_stopping_rounds=10               
             ).fit(dt_woe_bm,y)
         
         
-        hgirdTuner(XGBClassifier,para_space={
+        hgridTuner(XGBClassifier,para_space={
                   'n_estimators':sp_randint(low=60,high=120),#迭代次数
                   'learning_rate':sp_uniform(loc=0.05,scale=0.15), #学习率
                  
@@ -751,11 +722,11 @@ class test:
                   'reg_lambda':sp_randint(low=0,high=1), 
                   'scale_pos_weight':sp_uniform(loc=1,scale=0), 
                   'max_delta_step':sp_uniform(loc=0,scale=0)
-                  } ,method='h_random'           
+                  } ,method='h_random',early_stopping_rounds=10        
             ).fit(dt_woe_bm,y)   
         
         
-        hgirdTuner(LGBMClassifier,para_space={
+        hgridTuner(LGBMClassifier,para_space={
                      'boosting_type':['gbdt','goss'], 
                      'n_estimators':[100],
                      'learning_rate':[0.1], 
@@ -768,11 +739,11 @@ class test:
                      'subsample':[0.6,0.8],
                      'colsample_bytree' :[0.6,0.8],
                      'reg_lambda':[0,10], 
-                             },method='h_gird'           
+                             },method='h_grid',early_stopping_rounds=10               
             ).fit(dt_woe_bm,y)
 
         
-        hgirdTuner(LGBMClassifier,para_space={
+        hgridTuner(LGBMClassifier,para_space={
                      'boosting_type':['gbdt','goss'], #'goss','gbdt'
                      'n_estimators':sp_randint(low=100,high=110),
                      'learning_rate':sp_uniform(loc=0.1,scale=0), 
@@ -786,10 +757,37 @@ class test:
                      'colsample_bytree' :sp_uniform(loc=0.5,scale=0.5),
                      'reg_lambda':sp_uniform(loc=0,scale=20),
 
-                  } ,method='h_random'           
+                  } ,method='h_random',early_stopping_rounds=10               
+            ).fit(dt_woe_bm,y)        
+
+        
+        hgridTuner(CatBoostClassifier,
+                         para_space={
+                             'nan_mode':['Min'],
+                             'n_estimators': [80, 100],
+                             'learning_rate': [0.03,0.05, 0.1],
+                             'max_depth': [2,3],
+                             'scale_pos_weight': [1],
+                             'subsample': [1],
+                             'colsample_bylevel': [1],
+                             'reg_lambda': [0]
+                         } ,method='h_grid',early_stopping_rounds=10               
             ).fit(dt_woe_bm,y)                
         
         
+        hgridTuner(CatBoostClassifier,para_space={         
+                             'nan_mode':['Min'],
+                             'n_estimators':sp_randint(low=100,high=110),
+                             'learning_rate':sp_uniform(loc=0.1,scale=0),                     
+                             'max_depth':sp_randint(low=2,high=4),#[0,∞],                     
+                             'scale_pos_weight':[1],
+                             'subsample':sp_uniform(loc=0.5,scale=0.5),
+                             'colsample_bylevel' :sp_uniform(loc=0.5,scale=0.5),
+                             'reg_lambda':sp_uniform(loc=0,scale=20),
+                             },
+                    method='h_random',early_stopping_rounds=10               
+            ).fit(dt_woe_bm,y)        
+        
         print("tunner test successfully")
 
         
@@ -856,8 +854,11 @@ class test:
         
         figs_score=perfEval(show_plot=('density',),title='g-credit score').plot(dt_score_bm['score'],y_true,group,sample_weight,figure_size=(6,6))
         
+        gtab=GainsTable().fit_report(dt_score_bm['score'],y_true,group)
+        
         print(figs)
         print(figs_score)
+        print(gtab)
         
         print('perfEval test successfully')            
         
